@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/config.c 1.11 1997/03/09 13:24:57 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/config.c 1.18 1997/05/13 18:24:57 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2.
   Configuration file management functions.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -37,10 +37,12 @@ typedef enum _ConfigType
 
  PortNumber,                    /*+ For port numbers (>0). +*/
  AgeDays,                       /*+ An age in days (can be -ve). +*/
+ FileSize,                      /*+ A file size (must be >=0). +*/
  Boolean,                       /*+ A boolean response yes/no 1/0 true/false. +*/
 
  String,                        /*+ For an arbitrary string (no whitespace). +*/
  DirName,                       /*+ For directory name values. +*/
+ FileExt,                       /*+ A file extension (string). +*/
  HostAndPort,                   /*+ For host name and port numbers. +*/
  Host                           /*+ For host names. +*/
 }
@@ -51,6 +53,7 @@ typedef struct _Entry
 {
  char *name;                    /*+ The name of the entry. +*/
  void *pointer;                 /*+ A pointer to the value of the entry. +*/
+ char list;                     /*+ Set to true if it is a list of KeyPairs. +*/
  ConfigType left_type;          /*+ The type of the left side of the equals sign. +*/
  ConfigType right_type;         /*+ The type of the right side of the equals sign. +*/
 }
@@ -102,12 +105,12 @@ int UseSyslog=0;
 char *PassWord=NULL;
 
 /*+ The entries in the StartUp section. +*/
-static Entry startup_entries[]={{"http-port"    ,(void*)&HTTP_Port    ,Fixed,PortNumber},
-                                {"wwwoffle-port",(void*)&WWWOFFLE_Port,Fixed,PortNumber},
-                                {"spool-dir"    ,(void*)&SpoolDir     ,Fixed,DirName},
-                                {"use-syslog"   ,(void*)&UseSyslog    ,Fixed,Boolean},
-                                {"password"     ,(void*)&PassWord     ,Fixed,String},
-                                {NULL           ,NULL                 ,   -1,     -1}};
+static Entry startup_entries[]={{"http-port"    ,(void*)&HTTP_Port    ,0,Fixed,PortNumber},
+                                {"wwwoffle-port",(void*)&WWWOFFLE_Port,0,Fixed,PortNumber},
+                                {"spool-dir"    ,(void*)&SpoolDir     ,0,Fixed,DirName   },
+                                {"use-syslog"   ,(void*)&UseSyslog    ,0,Fixed,Boolean   },
+                                {"password"     ,(void*)&PassWord     ,0,Fixed,String    },
+                                {NULL           ,NULL                 ,0,   -1,        -1}};
 
 /*+ The StartUp section. +*/
 static Section startup_section={"StartUp",startup_entries};
@@ -124,11 +127,15 @@ int IndexLatestDays=7;
 /*+ The option of a tag that can be added to the bottom of the spooled pages with the date and a refresh button. +*/
 int AddInfoRefresh=0;
 
+/*+ The option to always request the server for changes when it is the cache and online. +*/
+int RequestChanged=1;
+
 /*+ The entries in the Options section. +*/
-static Entry options_entries[]={{"fetch-images"     ,(void*)&FetchImages    ,Fixed,Boolean},
-                                {"index-latest-days",(void*)&IndexLatestDays,Fixed,AgeDays},
-                                {"add-info-refresh" ,(void*)&AddInfoRefresh ,Fixed,Boolean},
-                                {NULL               ,NULL                   ,   -1,     -1}};
+static Entry options_entries[]={{"fetch-images"     ,(void*)&FetchImages    ,0,Fixed,Boolean},
+                                {"index-latest-days",(void*)&IndexLatestDays,0,Fixed,AgeDays},
+                                {"add-info-refresh" ,(void*)&AddInfoRefresh ,0,Fixed,Boolean},
+                                {"request-changed"  ,(void*)&RequestChanged ,0,Fixed,Boolean},
+                                {NULL               ,NULL                   ,0,   -1,     -1}};
 
 /*+ The Options section. +*/
 static Section options_section={"Options",options_entries};
@@ -140,21 +147,34 @@ static Section options_section={"Options",options_entries};
 static KeyPair **LocalHost=NULL;
 
 /*+ The entries in the LocalHost section. +*/
-static Entry localhost_entries[]={{""  ,(void*)&LocalHost,Host,None},
-                                  {NULL,NULL             ,  -1,  -1}};
+static Entry localhost_entries[]={{""  ,(void*)&LocalHost,1,Host,None},
+                                  {NULL,NULL             ,0,  -1,  -1}};
 
 /*+ The LocalHost section. +*/
 static Section localhost_section={"LocalHost",localhost_entries};
 
 
+/* LocalNet section */
+
+/*+ The list of local network hostnames. +*/
+static KeyPair **LocalNet=NULL;
+
+/*+ The entries in the LocalNet section. +*/
+static Entry localnet_entries[]={{""  ,(void*)&LocalNet,1,Host,None},
+                                 {NULL,NULL            ,0,  -1,  -1}};
+
+/*+ The LocalNet section. +*/
+static Section localnet_section={"LocalNet",localnet_entries};
+
+
 /* AllowedConnect section */
 
 /*+ The list of allowed hostnames. +*/
 static KeyPair **AllowedConnect=NULL;
 
 /*+ The entries in the AllowedConnect section. +*/
-static Entry allowedconnect_entries[]={{""  ,(void*)&AllowedConnect,Host,None},
-                                       {NULL,NULL                  ,  -1,  -1}};
+static Entry allowedconnect_entries[]={{""  ,(void*)&AllowedConnect,1,Host,None},
+                                       {NULL,NULL                  ,0,  -1,  -1}};
 
 /*+ The AllowedConnect section. +*/
 static Section allowedconnect_section={"AllowedConnect",allowedconnect_entries};
@@ -162,17 +182,59 @@ static Section allowedconnect_section={"AllowedConnect",allowedconnect_entries};
 
 /* DontCache section */
 
-/*+ The list of non-cached hostnames. +*/
-static KeyPair **DontCache=NULL;
+/*+ The list of hosts not to cache. +*/
+static KeyPair **DontCacheHosts=NULL;
+
+/*+ The list of file extensions not to cache. +*/
+static KeyPair **DontCacheExts=NULL;
+
+/*+ The list of non-cached hostnames + pathnames. +*/
+static KeyPair **DontCacheHostPath=NULL;
 
 /*+ The entries in the DontCache section. +*/
-static Entry dontcache_entries[]={{""  ,(void*)&DontCache,Host,None},
-                                  {NULL,NULL             ,  -1,  -1}};
+static Entry dontcache_entries[]={{"host"    ,(void*)&DontCacheHosts   ,1,Fixed,Host   },
+                                  {"file-ext",(void*)&DontCacheExts    ,1,Fixed,FileExt},
+                                  {""        ,(void*)&DontCacheHostPath,1,Host ,DirName},
+                                  {NULL      ,NULL                     ,0,   -1,     -1}};
 
 /*+ The DontCache section. +*/
 static Section dontcache_section={"DontCache",dontcache_entries};
 
 
+/* DontGet section */
+
+/*+ The list of hosts not to get. +*/
+static KeyPair **DontGetHosts=NULL;
+
+/*+ The list of file extensions not to get. +*/
+static KeyPair **DontGetExts=NULL;
+
+/*+ The list of non-got hostnames + pathnames. +*/
+static KeyPair **DontGetHostPath=NULL;
+
+/*+ The entries in the DontGet section. +*/
+static Entry dontget_entries[]={{"host"    ,(void*)&DontGetHosts   ,1,Fixed,Host   },
+                                {"file-ext",(void*)&DontGetExts    ,1,Fixed,FileExt},
+                                {""        ,(void*)&DontGetHostPath,1,Host ,DirName},
+                                {NULL      ,NULL                   ,0,   -1,     -1}};
+
+/*+ The DontGet section. +*/
+static Section dontget_section={"DontGet",dontget_entries};
+
+
+/* CensorHeader section */
+
+/*+ The list of censored headers. +*/
+static KeyPair **CensorHeader=NULL;
+
+/*+ The entries in the censor headers section. +*/
+static Entry censorheader_entries[]={{""  ,(void*)&CensorHeader,1,String,None},
+                                     {NULL,NULL                ,0,    -1,  -1}};
+
+/*+ The CensorHeader section. +*/
+static Section censorheader_section={"CensorHeader",censorheader_entries};
+
+
 /* Proxy section */
 
 /*+ The default proxy server to use. +*/
@@ -182,9 +244,9 @@ static char *DefaultProxy=NULL;
 static KeyPair **Proxies=NULL;
 
 /*+ The entries in the Proxy section. +*/
-static Entry proxy_entries[]={{"default",(void*)&DefaultProxy,Fixed,HostAndPort},
-                              {""       ,(void*)&Proxies     ,Host ,HostAndPort},
-                              {NULL     ,NULL                ,   -1,         -1}};
+static Entry proxy_entries[]={{"default",(void*)&DefaultProxy,0,Fixed,HostAndPort},
+                              {""       ,(void*)&Proxies     ,1,Host ,HostAndPort},
+                              {NULL     ,NULL                ,0,   -1,         -1}};
 
 /*+ The Proxy section. +*/
 static Section proxy_section={"Proxy",proxy_entries};
@@ -196,16 +258,20 @@ static Section proxy_section={"Proxy",proxy_entries};
 int PurgeUseMTime=0;
 
 /*+ The default age for purging files. +*/
-static int DefaultPurgeAge=DEF_PURGE_AGE;
+int DefaultPurgeAge=DEF_PURGE_AGE;
+
+/*+ The maximum allowed size of the cache. +*/
+int PurgeCacheSize=0;
 
 /*+ The list of hostnames and purge ages. +*/
 static KeyPair **PurgeAges=NULL;
 
 /*+ The entries in the Purge section. +*/
-static Entry purge_entries[]={{"use-mtime",(void*)&PurgeUseMTime  ,Fixed,Boolean},
-                              {"default"  ,(void*)&DefaultPurgeAge,Fixed,AgeDays},
-                              {""         ,(void*)&PurgeAges      ,Host ,AgeDays},
-                              {NULL       ,NULL                   ,   -1,     -1}};
+static Entry purge_entries[]={{"use-mtime",(void*)&PurgeUseMTime  ,0,Fixed,Boolean },
+                              {"default"  ,(void*)&DefaultPurgeAge,0,Fixed,AgeDays },
+                              {"max-size" ,(void*)&PurgeCacheSize ,0,Fixed,FileSize},
+                              {""         ,(void*)&PurgeAges      ,1,Host ,AgeDays },
+                              {NULL       ,NULL                   ,0,   -1,      -1}};
 
 /*+ The Purge section. +*/
 static Section purge_section={"Purge",purge_entries};
@@ -217,14 +283,19 @@ static Section purge_section={"Purge",purge_entries};
 static Section *sections[]={&startup_section,
                             &options_section,
                             &localhost_section,
+                            &localnet_section,
                             &allowedconnect_section,
                             &dontcache_section,
+                            &dontget_section,
+                            &censorheader_section,
                             &proxy_section,
                             &purge_section,
                             NULL};
 
 
 /* Local functions */
+static int NotCachedOrGot(char *host,char *path,KeyPair **Hosts,KeyPair **HostPath,KeyPair **Exts);
+
 static int ReadFile(FILE *config);
 static int ReadSection(FILE *config,Section *section);
 static int ParseEntry(Entry *entry,char *left,char *right);
@@ -344,6 +415,43 @@ int IsLocalHost(char *host)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Check if the specified hostname is to be cached.
+
+  int IsLocalNetHost Return true if the host is on the local network.
+
+  char *host The name of the host (and port number) to be checked.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int IsLocalNetHost(char *host)
+{
+ KeyPair **p;
+ char *colon=strchr(host,':');
+ int isip=(*host>='0' && *host<='9')?1:0;
+ int isit=0;
+
+ if(colon)
+    *colon=0;
+
+ if(LocalHost)
+    for(p=LocalHost;(*p)->name;p++)
+       if(!strcmp((*p)->name,host))
+         {isit=1;break;}
+
+ if(LocalNet && !isit)
+    for(p=LocalNet;(*p)->name;p++)
+       if(strlen((*p)->name)<=strlen(host) &&
+          ((!isip && !strcmp ((*p)->name,host+strlen(host)-strlen((*p)->name))) ||
+           ( isip && !strncmp((*p)->name,host,strlen((*p)->name)))))
+         {isit=1;break;}
+
+ if(colon)
+    *colon=':';
+
+ return(isit);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Check if the specified hostname is allowed to connect.
 
@@ -355,7 +463,7 @@ int IsLocalHost(char *host)
 int IsAllowedConnect(char *host)
 {
  KeyPair **p;
- int isip=(*host>=0 && *host<=9)?1:0;
+ int isip=(*host>='0' && *host<='9')?1:0;
  int isit=0;
 
  if(LocalHost)
@@ -375,37 +483,136 @@ int IsAllowedConnect(char *host)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Check if the specified hostname is to be cached.
+  Check if the specified URL is to be cached.
 
-  int IsCachedHost Return true if it is to be cached.
+  int IsNotCached Return true if it is not to be cached.
 
-  char *host The name of the host (and port number) to be checked.
+  char *host The name of the host to be checked.
+
+  char *path The path on the host to be checked.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int IsCachedHost(char *host)
+int IsNotCached(char *host,char *path)
+{
+ return(NotCachedOrGot(host,path,DontCacheHosts,DontCacheHostPath,DontCacheExts));
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Check if the specified URL is to be got.
+
+  int IsNotGot Return true if it is not to be got.
+
+  char *host The name of the host (and port number) to be checked.
+
+  char *path The path on the host to be checked.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int IsNotGot(char *host,char *path)
+{
+ return(NotCachedOrGot(host,path,DontGetHosts,DontGetHostPath,DontGetExts));
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Check if the URL is not to be cached or got based on the keypair lists.
+
+  int NotCachedOrGot Return true if it is not to be cached or got.
+
+  char *host The host name to check.
+
+  char *path The path to check.
+
+  KeyPair **Hosts The list of hostss not to cache or get.
+
+  KeyPair **HostPath The list of hosts and paths not to cache or get.
+
+  KeyPair **Exts The list of file extensions not to cache or get.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int NotCachedOrGot(char *host,char *path,KeyPair **Hosts,KeyPair **HostPath,KeyPair **Exts)
 {
  KeyPair **p;
- char *colon=strchr(host,':');
- int isip=(*host>=0 && *host<=9)?1:0;
- int isit=1;
+ int isit=0;
 
- if(colon)
-    *colon=0;
+ if(Hosts)
+   {
+    char *colon=strchr(host,':');
+    int isip=(*host>='0' && *host<='9')?1:0;
 
- if(LocalHost)
-    for(p=LocalHost;(*p)->name;p++)
-       if(!strcmp((*p)->name,host))
-         {isit=0;break;}
+    if(colon)
+       *colon=0;
 
- if(DontCache && isit)
-    for(p=DontCache;(*p)->name;p++)
+    for(p=Hosts;(*p)->name;p++)
        if(strlen((*p)->name)<=strlen(host) &&
           ((!isip && !strcmp ((*p)->name,host+strlen(host)-strlen((*p)->name))) ||
            ( isip && !strncmp((*p)->name,host,strlen((*p)->name)))))
-         {isit=0;break;}
+         {isit=1;break;}
 
- if(colon)
-    *colon=':';
+    if(colon)
+       *colon=':';
+   }
+
+ if(HostPath && !isit)
+   {
+    char *colon=strchr(host,':');
+    int isip=(*host>='0' && *host<='9')?1:0;
+
+    if(colon)
+       *colon=0;
+
+    for(p=HostPath;(*p)->name;p++)
+       if(strlen((*p)->name)<=strlen(host) &&
+          ((!isip && !strcmp ((*p)->name,host+strlen(host)-strlen((*p)->name))) ||
+           ( isip && !strncmp((*p)->name,host,strlen((*p)->name)))))
+          if(!strncmp((*p)->value.string+1,path,strlen((*p)->value.string)-1))
+            {isit=1;break;}
+
+    if(colon)
+       *colon=':';
+   }
+
+ if(Exts && !isit)
+   {
+    char *ext=NULL;
+    int i;
+
+    for(i=strlen(path);i>=0;i--)
+       if(path[i]=='/')
+          break;
+       else if(path[i]=='.')
+         {ext=&path[i+1];break;}
+
+    if(!ext)
+       isit=0;
+    else
+       for(p=Exts;(*p)->value.string;p++)
+          if(!strcmp((*p)->value.string,ext))
+            {isit=1;break;}
+   }
+
+ return(isit);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Decide if the header line is to be sent to the server.
+
+  int IsCensoredheader Returns true if the header line is to be deleted.
+
+  char *line The line to check.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int IsCensoredHeader(char *line)
+{
+ KeyPair **p;
+ int isit=0;
+
+ if(CensorHeader && !isit)
+    for(p=CensorHeader;(*p)->name;p++)
+       if(line[strlen((*p)->name)]==':' &&
+          !strncmp((*p)->name,line,strlen((*p)->name)))
+         {isit=1;break;}
 
  return(isit);
 }
@@ -423,7 +630,7 @@ char *WhichProxy(char *host)
 {
  KeyPair **p;
  char *colon=strchr(host,':');
- int isip=(*host>=0 && *host<=9)?1:0;
+ int isip=(*host>='0' && *host<='9')?1:0;
  char *proxy=DefaultProxy;
  int matchlen=0;
 
@@ -460,7 +667,7 @@ int WhatPurgeAge(char *host)
 {
  KeyPair **p;
  char *colon=strchr(host,':');
- int isip=(*host>=0 && *host<=9)?1:0;
+ int isip=(*host>='0' && *host<='9')?1:0;
  int age=DefaultPurgeAge;
  int matchlen=0;
 
@@ -681,18 +888,16 @@ static int ReadSection(FILE *config,Section *section)
 
 static int ParseEntry(Entry *entry,char *left,char *right)
 {
- if(*entry->name)
-   {
-    if(ParseValue(right,entry->right_type,entry->pointer,1))
-       return(1);
-   }
- else
+ if(entry->list)
    {
     KeyPair **p,*new=(KeyPair*)malloc(sizeof(KeyPair));
     int i;
 
-    if(ParseValue(left,entry->left_type,(void*)&new->name,0))
-      {free(new);return(1);}
+    if(entry->left_type==Fixed)
+       new->name=entry->name;
+    else
+       if(ParseValue(left,entry->left_type,(void*)&new->name,0))
+         {free(new);return(1);}
 
     if(entry->right_type==None)
        new->value.string=NULL;
@@ -718,6 +923,9 @@ static int ParseEntry(Entry *entry,char *left,char *right)
     p++;
     *p=&KeyPairEnd;
    }
+ else
+    if(ParseValue(right,entry->right_type,entry->pointer,1))
+       return(1);
 
  return(0);
 }
@@ -755,6 +963,14 @@ static int ParseValue(char *text,ConfigType type,void *value,int rhs)
     *(int*)value=atoi(text);
     break;
 
+   case FileSize:
+    if(!*text)
+      {errmsg=(char*)malloc(48);strcpy(errmsg,"Expecting a file size in MB, got nothing.");break;}
+    if(atoi(text)<0)
+      {errmsg=(char*)malloc(48);sprintf(errmsg,"Invalid file size %d.",atoi(text));break;}
+    *(int*)value=atoi(text);
+    break;
+
    case Boolean:
     if(!*text)
        *(int*)value=0;
@@ -784,6 +1000,11 @@ static int ParseValue(char *text,ConfigType type,void *value,int rhs)
     strcpy(*(char**)value,text);
     break;
 
+   case FileExt:
+    *(char**)value=(char*)malloc(strlen(text)+1);
+    strcpy(*(char**)value,text);
+    break;
+
    case Host:
     if(rhs && (!*text || !strcasecmp(text,"none")))
        *(char**)value=NULL;
@@ -834,17 +1055,38 @@ static int old_FetchImages=0;
 /*+ The old value of the number of days to display in the index of the latest pages. +*/
 static int old_IndexLatestDays=0;
 
-/*+ A old value of the option for a tag that can be added to the bottom of the spooled pages with the date and a refresh button. +*/
+/*+ The old value of the option for a tag that can be added to the bottom of the spooled pages with the date and a refresh button. +*/
 static int old_AddInfoRefresh=0;
 
+/*+ The old value of the option to always request the server for changes when it is the cache and online. +*/
+static int old_RequestChanged=1;
+
 /*+ The old values of the list of localhost hostnames. +*/
 static KeyPair **old_LocalHost=NULL;
 
+/*+ The old value of the list of local network hostnames. +*/
+static KeyPair **old_LocalNet=NULL;
+
 /*+ The old values of the list of allowed hostnames. +*/
 static KeyPair **old_AllowedConnect=NULL;
 
+/*+ The old value of the list of hosts not to cache. +*/
+static KeyPair **old_DontCacheHosts=NULL;
+
+/*+ The old value of the list of file extensions not to cache. +*/
+static KeyPair **old_DontCacheExts=NULL;
+
 /*+ The old value of the list of non-cached hostnames. +*/
-static KeyPair **old_DontCache=NULL;
+static KeyPair **old_DontCacheHostPath=NULL;
+
+/*+ The old value of the list of hosts not to get. +*/
+static KeyPair **old_DontGetHosts=NULL;
+
+/*+ The old value of the list of file extensions not to get. +*/
+static KeyPair **old_DontGetExts=NULL;
+
+/*+ The old value of the list of non-got hostnames. +*/
+static KeyPair **old_DontGetHostPath=NULL;
 
 /*+ The old value of the default proxy server to use. +*/
 static char *old_DefaultProxy=NULL;
@@ -858,6 +1100,9 @@ static int old_PurgeUseMTime=0;
 /*+ The old value of the default age for purging files. +*/
 static int old_DefaultPurgeAge=0;
 
+/*+ The old value of the maximum allowed size of the cache. +*/
+static int old_PurgeCacheSize=0;
+
 /*+ The old value of the list of hostnames and purge ages. +*/
 static KeyPair **old_PurgeAges=NULL;
 
@@ -869,17 +1114,36 @@ static KeyPair **old_PurgeAges=NULL;
 static void SaveOldValues(void)
 {
  old_FetchImages=FetchImages;
+ FetchImages=0;
  old_IndexLatestDays=IndexLatestDays;
+ IndexLatestDays=7;
  old_AddInfoRefresh=AddInfoRefresh;
+ AddInfoRefresh=0;
+ old_RequestChanged=RequestChanged;
+ RequestChanged=1;
 
  old_LocalHost=LocalHost;
  LocalHost=NULL;
 
+ old_LocalNet=LocalNet;
+ LocalNet=NULL;
+
  old_AllowedConnect=AllowedConnect;
  AllowedConnect=NULL;
 
- old_DontCache=DontCache;
- DontCache=NULL;
+ old_DontCacheHosts=DontCacheHosts;
+ DontCacheHosts=NULL;
+ old_DontCacheExts=DontCacheExts;
+ DontCacheExts=NULL;
+ old_DontCacheHostPath=DontCacheHostPath;
+ DontCacheHostPath=NULL;
+
+ old_DontGetHosts=DontGetHosts;
+ DontGetHosts=NULL;
+ old_DontGetExts=DontGetExts;
+ DontGetExts=NULL;
+ old_DontGetHostPath=DontGetHostPath;
+ DontGetHostPath=NULL;
 
  old_DefaultProxy=DefaultProxy;
  DefaultProxy=NULL;
@@ -887,7 +1151,11 @@ static void SaveOldValues(void)
  Proxies=NULL;
 
  old_PurgeUseMTime=PurgeUseMTime;
+ PurgeUseMTime=0;
  old_DefaultPurgeAge=DefaultPurgeAge;
+ DefaultPurgeAge=DEF_PURGE_AGE;
+ old_PurgeCacheSize=PurgeCacheSize;
+ PurgeCacheSize=0;
  old_PurgeAges=PurgeAges;
  PurgeAges=NULL;
 }
@@ -902,30 +1170,52 @@ static void RestoreOldValues(void)
  FetchImages=old_FetchImages;
  IndexLatestDays=old_IndexLatestDays;
  AddInfoRefresh=old_AddInfoRefresh;
+ RequestChanged=old_RequestChanged;
 
  if(LocalHost)
-    FreeKeyPairList(LocalHost,0);
+    FreeKeyPairList(LocalHost,1);
  LocalHost=old_LocalHost;
 
+ if(LocalNet)
+    FreeKeyPairList(LocalNet,1);
+ LocalNet=old_LocalNet;
+
  if(AllowedConnect)
-    FreeKeyPairList(AllowedConnect,0);
+    FreeKeyPairList(AllowedConnect,1);
  AllowedConnect=old_AllowedConnect;
 
- if(DontCache)
-    FreeKeyPairList(DontCache,0);
- DontCache=old_DontCache;
+ if(DontCacheHosts)
+    FreeKeyPairList(DontCacheHosts,2);
+ DontCacheHosts=old_DontCacheHosts;
+ if(DontCacheExts)
+    FreeKeyPairList(DontCacheExts,2);
+ DontCacheExts=old_DontCacheExts;
+ if(DontCacheHostPath)
+    FreeKeyPairList(DontCacheHostPath,3);
+ DontCacheHostPath=old_DontCacheHostPath;
+
+ if(DontGetHosts)
+    FreeKeyPairList(DontGetHosts,2);
+ DontGetHosts=old_DontGetHosts;
+ if(DontGetExts)
+    FreeKeyPairList(DontGetExts,2);
+ DontGetExts=old_DontGetExts;
+ if(DontGetHostPath)
+    FreeKeyPairList(DontGetHostPath,3);
+ DontGetHostPath=old_DontGetHostPath;
 
  if(DefaultProxy)
     free(DefaultProxy);
  DefaultProxy=old_DefaultProxy;
  if(Proxies)
-    FreeKeyPairList(Proxies,1);
+    FreeKeyPairList(Proxies,3);
  Proxies=old_Proxies;
 
  PurgeUseMTime=old_PurgeUseMTime;
  DefaultPurgeAge=old_DefaultPurgeAge;
+ PurgeCacheSize=old_PurgeCacheSize;
  if(PurgeAges)
-    FreeKeyPairList(PurgeAges,0);
+    FreeKeyPairList(PurgeAges,1);
  PurgeAges=old_PurgeAges;
 }
 
@@ -937,21 +1227,35 @@ static void RestoreOldValues(void)
 static void RemoveOldValues(void)
 {
  if(old_LocalHost)
-    FreeKeyPairList(old_LocalHost,0);
+    FreeKeyPairList(old_LocalHost,1);
+
+ if(old_LocalNet)
+    FreeKeyPairList(old_LocalNet,1);
 
  if(old_AllowedConnect)
-    FreeKeyPairList(old_AllowedConnect,0);
+    FreeKeyPairList(old_AllowedConnect,1);
 
- if(old_DontCache)
-    FreeKeyPairList(old_DontCache,0);
+ if(old_DontCacheHosts)
+    FreeKeyPairList(old_DontCacheHosts,2);
+ if(old_DontCacheExts)
+    FreeKeyPairList(old_DontCacheExts,2);
+ if(old_DontCacheHostPath)
+    FreeKeyPairList(old_DontCacheHostPath,3);
+
+ if(old_DontGetHosts)
+    FreeKeyPairList(old_DontGetHosts,2);
+ if(old_DontGetExts)
+    FreeKeyPairList(old_DontGetExts,2);
+ if(old_DontGetHostPath)
+    FreeKeyPairList(old_DontGetHostPath,3);
 
  if(old_DefaultProxy)
     free(old_DefaultProxy);
  if(old_Proxies)
-    FreeKeyPairList(old_Proxies,1);
+    FreeKeyPairList(old_Proxies,3);
 
  if(old_PurgeAges)
-    FreeKeyPairList(old_PurgeAges,0);
+    FreeKeyPairList(old_PurgeAges,1);
 }
 
 
@@ -960,17 +1264,18 @@ static void RemoveOldValues(void)
 
   KeyPair **list The list to free.
 
-  int andvalue If true then free the value field as well.
+  int freewhat If 1 then free the name, if 2 then free the value, if 3 free both, if 0 free neither.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void FreeKeyPairList(KeyPair **list,int andvalue)
+static void FreeKeyPairList(KeyPair **list,int freewhat)
 {
  KeyPair **p;
 
  for(p=list;(*p)->name;p++)
    {
-    free((*p)->name);
-    if(andvalue && (*p)->value.string)
+    if(freewhat&1)
+       free((*p)->name);
+    if(freewhat&2 && (*p)->value.string)
        free((*p)->value.string);
     free(*p);
    }
