@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/index.c 1.14 1997/03/17 17:08:59 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/index.c 1.19 1997/06/23 20:16:14 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
   Generate an index of the web pages that are cached in the system.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -109,7 +109,7 @@ void IndexPage(int fd,char *path,char *args)
  "</HTML>\n";
  SortMode sort=NSortModes;
  char *newpath=(char*)malloc(strlen(path)+1);
- char *buffer=(char*)malloc(strlen(path)+64);
+ char *buffer=(char*)malloc(strlen(path)+80);
  struct stat buf;
 
  strcpy(newpath,path);
@@ -157,7 +157,7 @@ void IndexPage(int fd,char *path,char *args)
 
        if(sort==Latest)
          {
-          sprintf(buffer,"<a href=\"/index/?none\">[Back to the Hosts Index]</a>\n");
+          sprintf(buffer,"<p align=center><a href=\"/index/?none\">[Back to the Hosts Index]</a></p>\n");
           write(fd,buffer,strlen(buffer));
          }
       }
@@ -168,7 +168,7 @@ void IndexPage(int fd,char *path,char *args)
 
        IndexDir(fd,newpath,sort);
 
-       sprintf(buffer,"</ul>\n<a href=\"/index/?%s\">[Back to the Hosts Index]</a>\n",args);
+       sprintf(buffer,"</ul>\n<p align=center><a href=\"/index/?%s\">[Back to the Hosts Index]</a></p>\n",args);
        write(fd,buffer,strlen(buffer));
       }
 
@@ -236,6 +236,7 @@ static void IndexRoot(int fd,char *args,SortMode mode)
     write(fd,buffer,strlen(buffer));
 
     free(buffer);
+    free(files[i]->name);
     free(files[i]);
    }
 
@@ -246,6 +247,8 @@ static void IndexRoot(int fd,char *args,SortMode mode)
    }
 
  free(files);
+ files=NULL;
+ nfiles=0;
 }
 
 
@@ -306,20 +309,35 @@ static void IndexDir(int fd,char *dirname,SortMode mode)
 
  for(i=0;i<nfiles;i++)
    {
-    char *buffer=(char*)malloc(3*strlen(files[i]->name)+2*strlen(dirname)+128);
+    char *buffer=(char*)malloc(4*strlen(files[i]->name)+3*strlen(dirname)+160);
+    char *url=(char*)malloc(strlen(files[i]->name)+strlen(dirname)+8),*encurl;
+
+    strcpy(url,dirname);
+    strcat(url,"/");
+    if(files[i]->name[0]=='/')
+       strcat(url,&files[i]->name[1]);
+    else
+       strcat(url,files[i]->name);
+    encurl=UrlEncode(url);
 
     if(strstr(files[i]->name,"?!"))
-       sprintf(buffer,"<li>[Posted]&nbsp;&nbsp;&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
+       sprintf(buffer,"<li>[PostedNoRefresh]"
+                      "&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
                dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
                files[i]->name);
     else
-       sprintf(buffer,"<li><a href=\"/outgoing/%s/%s\">[Refresh]</a>&nbsp;&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
+       sprintf(buffer,"<li>[<a href=\"/refresh/%s/%s\">Refresh</a>|"
+                      "<a href=\"/refresh/?%s\">Options</a>]"
+                      "&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
                dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
-               dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
-               files[i]->name);
+               encurl,
+               dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,files[i]->name);
     write(fd,buffer,strlen(buffer));
 
+    free(url);
+    free(encurl);
     free(buffer);
+    free(files[i]->name);
     free(files[i]);
    }
 
@@ -330,13 +348,15 @@ static void IndexDir(int fd,char *dirname,SortMode mode)
    }
 
  free(files);
+ files=NULL;
+ nfiles=0;
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
   Create an index of the latest pages.
 
-  int fd Thefile descriptor to write to.
+  int fd The file descriptor to write to.
   ++++++++++++++++++++++++++++++++++++++*/
 
 static void IndexLatest(int fd)
@@ -364,10 +384,10 @@ static void IndexLatest(int fd)
    {
     struct stat buf;
 
-    if(stat(ent->d_name,&buf))
+    if(lstat(ent->d_name,&buf))
        PrintMessage(Inform,"Cannot stat file '%s' [%!s]; race condition?",ent->d_name);
     else
-       if(S_ISDIR(buf.st_mode) && strcmp(ent->d_name,"outgoing"))
+       if(S_ISDIR(buf.st_mode) && strcmp(ent->d_name,"outgoing") && (now-buf.st_mtime)<=IndexLatestDays*(24*3600))
          {
           DIR *dir2;
           struct dirent* ent2;
@@ -412,7 +432,8 @@ static void IndexLatest(int fd)
 
  for(i=0;i<nfiles;i++)
    {
-    char *buffer=(char*)malloc(3*strlen(files[i]->name)+128);
+    char *encurl=UrlEncode(files[i]->name);
+    char *buffer=(char*)malloc(3*strlen(files[i]->name)+strlen(encurl)+160);
     int days=(now-files[i]->time)/(24*3600);
 
     if(lastdays<days)
@@ -429,14 +450,21 @@ static void IndexLatest(int fd)
       }
 
     if(strstr(files[i]->name,"?!"))
-       sprintf(buffer,"<li>[Posted]&nbsp;&nbsp;&nbsp;<a href=\"http://%s\">%s</a>\n",
+       sprintf(buffer,"<li>[PostedNoRefresh]"
+                      "&nbsp;<a href=\"http://%s\">%s</a>\n",
                files[i]->name,files[i]->name);
     else
-       sprintf(buffer,"<li><a href=\"/outgoing/%s\">[Refresh]</a>&nbsp;&nbsp;<a href=\"http://%s\">%s</a>\n",
-               files[i]->name,files[i]->name,files[i]->name);
+       sprintf(buffer,"<li>[<a href=\"/refresh/%s\">Refresh</a>|"
+                      "<a href=\"/refresh/?%s\">Options</a>]"
+                      "&nbsp;<a href=\"http://%s\">%s</a>\n",
+               files[i]->name,
+               encurl,
+               files[i]->name,files[i]->name);
     write(fd,buffer,strlen(buffer));
 
+    free(encurl);
     free(buffer);
+    free(files[i]->name);
     free(files[i]);
    }
 
@@ -454,6 +482,8 @@ static void IndexLatest(int fd)
    }
 
  free(files);
+ files=NULL;
+ nfiles=0;
 }
 
 
@@ -678,6 +708,8 @@ static void add_file(char *name,struct stat *buf,SortMode mode,char *dir)
           sprintf(files[nfiles]->name,"%s/%s?%s",dir,*name=='/'?"":name,args);
        else
           sprintf(files[nfiles]->name,"%s?%s",name,args);
+
+       free(args);
       }
     else
       {
