@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 1.11 1997/03/24 10:06:13 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 1.16 1997/06/26 15:57:58 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
   A demon program to maintain the database and spawn the servers.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -50,11 +50,21 @@ int online=0;
 int n_servers=0,                /*+ in total. +*/
     n_fetch_servers=0;          /*+ fetching a page. +*/
 
+/*+ The wwwoffle client file descriptor when fetching. +*/
+int fetch_fd=-1;
+
 /*+ The pids of the servers that are fetching. +*/
 int fetch_pids[MAX_FETCH_SERVERS];
 
-/*+ The wwwoffle client file descriptor when fetching. +*/
-int fetch_fd=0;
+/*+ The current status, fetching or not. +*/
+int fetching=0;
+
+
+/*+ The pid of a child that has exited. +*/
+static int exit_pid[MAX_SERVERS];
+
+/*+ The status of the child that has exited. +*/
+static int exit_status[MAX_SERVERS];
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -91,12 +101,15 @@ int main(int argc, char** argv)
  for(i=0;i<MAX_FETCH_SERVERS;i++)
     fetch_pids[i]=0;
 
+ for(i=0;i<MAX_SERVERS;i++)
+    exit_pid[i]=exit_status[i]=0;
+
  InitErrorHandler("wwwoffled",1);
 
  if(ReadConfigFile(2))
     PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigFile);
 
- PrintMessage(Important,"WWWOFFLE Demon Version 1.1 started");
+ PrintMessage(Important,"WWWOFFLE Demon Version 1.2b started");
  PrintMessage(Inform,"WWWOFFLE Read Configuration File.");
 
  http_fd=OpenServerSocket(HTTP_Port);
@@ -129,6 +142,7 @@ int main(int argc, char** argv)
 
  do
    {
+    struct timeval tv;
     fd_set readfd;
     int nfds;
     int retval;
@@ -145,7 +159,10 @@ int main(int argc, char** argv)
     if(n_servers<MAX_SERVERS)
        FD_SET(http_fd,&readfd);
 
-    retval=select(nfds,&readfd,NULL,NULL,NULL);
+    tv.tv_sec=10;
+    tv.tv_usec=0;
+
+    retval=select(nfds,&readfd,NULL,NULL,&tv);
 
     if(retval!=-1)
       {
@@ -157,9 +174,7 @@ int main(int argc, char** argv)
           client=AcceptConnect(wwwoffle_fd,&host,&port);
           if(client>=0)
             {
-             int allowed=IsAllowedConnect(host);
-
-             if(allowed)
+             if(IsAllowedConnect(host))
                {
                 FILE *file=fdopen(client,"r");
 
@@ -168,7 +183,6 @@ int main(int argc, char** argv)
                 if(!file)
                   {
                    PrintMessage(Warning,"Unable to fdopen the wwwoffle control socket [%!s].");
-
                    CloseSocket(client);
                   }
                 else
@@ -182,7 +196,6 @@ int main(int argc, char** argv)
              else
                {
                 PrintMessage(Warning,"WWWOFFLE connection from %s rejected.",host);
-
                 CloseSocket(client);
                }
             }
@@ -196,12 +209,9 @@ int main(int argc, char** argv)
           client=AcceptConnect(http_fd,&host,&port);
           if(client>=0)
             {
-             int allowed=IsAllowedConnect(host);
-
-             if(allowed)
+             if(IsAllowedConnect(host))
                {
                 PrintMessage(Inform,"HTTP Proxy connection from %s.",host);
-
                 ForkServer(client,1);
                }
              else
@@ -211,6 +221,35 @@ int main(int argc, char** argv)
             }
          }
       }
+
+    for(i=0;i<MAX_SERVERS;i++)
+       if(exit_pid[i]!=0)
+         {
+          if(exit_status[i]>=0)
+             PrintMessage(Inform,"Child wwwoffles exited with status %d (pid=%d).",exit_status[i],exit_pid[i]);
+          else
+             PrintMessage(Important,"Child wwwoffles terminated by signal %d (pid=%d).",-exit_status[i],exit_pid[i]);
+          exit_pid[i]=exit_status[i]=0;
+         }
+
+    if(fetch_fd!=-1)
+      {
+       if(!online)
+          fetching=0;
+
+       while(fetching && n_fetch_servers<MAX_FETCH_SERVERS && n_servers<MAX_SERVERS)
+          ForkServer(fetch_fd,0);
+
+       if(!fetching && n_fetch_servers==0)
+         {
+          char *msg="WWWOFFLE No more to fetch.\n";
+          write(fetch_fd,msg,strlen(msg));
+          close(fetch_fd);
+          fetch_fd=-1;
+
+          PrintMessage(Important,"WWWOFFLE Fetch finished.");
+         }
+      }
    }
  while(!closedown);
 
@@ -232,7 +271,7 @@ int main(int argc, char** argv)
 static void usage(int verbose)
 {
  fputs("\n"
-       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 1.1\n"
+       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 1.2b\n"
        "\n",stderr);
 
  if(verbose)
@@ -274,62 +313,48 @@ static void sigchild(int signum)
        int exitval=0;
 
        if(WIFEXITED(status))
-         {
           exitval=WEXITSTATUS(status);
-
-          PrintMessage(Inform,"Child wwwoffles exited with status %d (pid=%d).",exitval,pid);
-         }
        else if(WIFSIGNALED(status))
-         {
-          int signal=WTERMSIG(status);
+          exitval=-WTERMSIG(status);
 
-          PrintMessage(Inform,"Child wwwoffles terminated by signal %d (pid=%d).",signal,pid);
-         }
-       else
-          PrintMessage(Inform,"Child wwwoffles finished (pid=%d).",pid);
+       for(i=0;i<MAX_SERVERS;i++)
+          if(exit_pid[i]==0)
+            {
+             exit_pid[i]=pid;
+             exit_status[i]=exitval;
+             break;
+            }
 
        n_servers--;
 
-       for(i=0;i<MAX_FETCH_SERVERS;i++)
-          if(fetch_pids[i]==pid)
-             break;
-
-       if(i!=MAX_FETCH_SERVERS)
+       if(fetch_fd!=-1)
          {
-          n_fetch_servers--;
-          fetch_pids[i]=0;
+          for(i=0;i<MAX_FETCH_SERVERS;i++)
+             if(fetch_pids[i]==pid)
+                break;
 
-          if(fetch_fd>0 && exitval==3)
-             fetch_fd=-fetch_fd;
-
-          if(fetch_fd<0 && exitval==4)
-             fetch_fd=-fetch_fd;
-
-          if(fetch_fd<0 && n_fetch_servers==0)
+          if(i!=MAX_FETCH_SERVERS)
             {
-             char *msg="WWWOFFLE No more to fetch.\n";
-             write(-fetch_fd,msg,strlen(msg));
+             n_fetch_servers--;
+             fetch_pids[i]=0;
 
-             PrintMessage(Important,"WWWOFFLE Fetch finished.");
+             if(exitval==3)
+                fetching=0;
 
-             close(-fetch_fd);
-             fetch_fd=0;
+             if(exitval==4 && online)
+                fetching=1;
             }
          }
-
-       if(fetch_fd>0)
-          for(;n_fetch_servers<MAX_FETCH_SERVERS && n_servers<MAX_SERVERS;)
-             ForkServer(fetch_fd,0);
       }
 
 #if defined(SIGCHLD)
  sig=signal(SIGCHLD,sigchild);
  if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGGHLD handler.");
+    PrintMessage(Fatal,"Cannot add SIGGHLD handler.");
 #else
  sig=signal(SIGCLD,sigchild);
  if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGGLD handler.");
+    PrintMessage(Fatal,"Cannot add SIGGLD handler.");
 #endif
 }
 
@@ -348,6 +373,7 @@ static void sigexit(int signum)
    {
     closedown=1;
 
+    /* Ignore the PrintMessage re-entrancy problem since we are exiting anyway. */
     PrintMessage(Important,"Exit signalled.");
    }
 
