@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffles.c 1.25 1997/03/24 17:36:51 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffles.c 1.42 1997/06/26 16:07:32 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
   A server to fetch the required pages.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -42,7 +42,7 @@ typedef enum _Mode
  Spool,                         /*+ From cache to client. +*/
  SpoolGet,                      /*+ Not in cache so record request in outgoing. +*/
  SpoolRefresh,                  /*+ Refresh the page, forced from index. +*/
- Local                          /*+ A local page e.g. the index. +*/
+ SpoolPragma                    /*+ Refresh the page, forced from browser by 'Pragma: no-cache'. +*/
 }
 Mode;
 
@@ -112,11 +112,7 @@ int wwwoffles(int online,int browser,int client)
    {
     outgoing=OpenOutgoingSpoolFile(1);
 
-    PrintMessage(Debug,"outgoing=%d",outgoing);
-
     if(outgoing==-1)
-      {PrintMessage(Warning,"Cannot open the spooled outgoing request to read.");exit(1);}
-    else if(outgoing==-2)
       {PrintMessage(Inform,"No more outgoing requests.");exit(3);}
 
     outgoing_file=fdopen(outgoing,"r");
@@ -133,7 +129,8 @@ int wwwoffles(int online,int browser,int client)
 
  if(!url)
    {
-    ServerError(client,request?"Cannot parse the HTTP request":"The HTTP request was empty");
+    if(mode!=Fetch)
+       ServerError(client,request?"Cannot parse the HTTP request":"The HTTP request was empty");
     PrintMessage(Warning,"Could not parse HTTP request (%s).",request?"Parse error":"Empty request");
     exit(1);
    }
@@ -143,75 +140,134 @@ int wwwoffles(int online,int browser,int client)
  SplitURL(url,&host,&path,&args);
 
  /* Change the mode based on the URL as required. */
+ /* (At this point mode is Spool, Real or Fetch only.) */
 
- if((mode==Real || mode==Spool) && !*host)
+ if(IsNotGot(host,path))
    {
-    if(!strncmp(path,"outgoing/",9))
+    char* colon=strchr(host,':');
+    if(colon) *colon=0;
+    if(mode==Real || mode==Spool)
+       RemoteHostError(client,url,"is on the list of hosts and paths not to get.");
+    PrintMessage(Inform,"The host '%s' and/or path '%s' is on the list not to get.",host,path);
+    exit(0);
+   }
+ else if(!*host)
+   {
+    int newurl=0;
+
+    if(!strncmp("refresh",path,7))
       {
-       char *copy,*u,*p;
+       url=RefreshPage(client,path,args,url,&request,online);
 
-       copy=(char*)malloc(strlen(request)+1);
-       p=strstr(request,path)+9;
-       u=strstr(request,url); *u=0;
-       strcpy(copy,request);
-       strcat(copy,"http://");
-       strcat(copy,p);
-       free(request);
-       request=copy;
-
-       copy=(char*)malloc(strlen(url)+1);
-       p=strstr(url,path)+9;
-       strcpy(copy,"http://");
-       strcat(copy,p);
-       free(url);
-       url=copy;
-
-       SplitURL(url,&host,&path,&args);
-
-       if(!*host || !IsCachedHost(host))
-          mode=RealNoCache;
-       else if(mode==Real)
+       if(url)
          {
-          mode=RealRefresh;
-          OpenWebpageSpoolFile(-1,host,path,args);
-         }
-       else if(mode==Spool)
-         {
-          spool=OpenWebpageSpoolFile(1,host,path,args);
-          if(spool!=-1)
-            {mode=SpoolRefresh;close(spool);}
-          else
-             mode=SpoolGet;
+          SplitURL(url,&host,&path,&args);
+
+          if(args && *args=='!')
+            {
+             PrintMessage(Inform,"It is not possible to refresh a URL that was posted.");
+             if(mode!=Fetch)
+                CantRefreshPosted(client,url);
+             exit(0);
+            }
+          else if(!*host || IsLocalNetHost(host))
+             mode=RealNoCache;
+          else if(mode==Real)
+            {
+             mode=RealRefresh;
+             OpenWebpageSpoolFile(-1,host,path,args);
+            }
+          else if(mode==Spool)
+            {
+             spool=OpenWebpageSpoolFile(1,host,path,args);
+             if(spool!=-1)
+               {mode=SpoolRefresh;close(spool);}
+             else
+                mode=SpoolGet;
+            }
+          newurl=1;
          }
       }
+    else if(mode==Fetch)
+       PrintMessage(Inform,"The request to fetch a page from the local host is ignored.");
+    else if(!*path && !args)
+       WelcomePage(client,PassWord?1:0);
+    else if(!strncmp("index/",path,6))
+       IndexPage(client,&path[6],args);
+    else if(!strncmp("control/",path,8))
+       ControlPage(client,&path[8],request);
     else
-       mode=Local;
+       IllegalPage(client,path,args);
+
+    if(!newurl)
+       exit(0);
    }
- else if((mode==Real || mode==Spool) && !IsCachedHost(host))
+ else if(IsLocalNetHost(host))
+   {
+    if(mode==Real || mode==Spool)
+       mode=RealNoCache;
+    else
+      {
+       PrintMessage(Inform,"The request to fetch a page from the local network host '%s' is ignored.",host);
+       exit(0);
+      }
+   }
+ else if(mode==Real && IsNotCached(host,path))
    {
     mode=RealNoCache;
    }
- else if(mode==Fetch && (!*host || !IsCachedHost(host)))
+ else if(!strncasecmp(request,"POST",4))
    {
-    PrintMessage(Inform,"A request to fetch a page from a non-cached host is ignored.");
-    exit(0);
+    if(mode==Spool)
+       mode=SpoolGet;
    }
- else if(mode==Real || mode==Spool || mode==Fetch)
+ else if(args && *args=='!')
+   {
+    if(mode==Real)
+       mode=Spool;
+    else if(mode==Fetch)
+      {
+       PrintMessage(Inform,"It is not possible to fetch a URL that was posted.");
+       exit(0);
+      }
+   }
+ else if(strstr(request,"\nPragma: no-cache"))
+   {
+    if(mode==Spool)
+      {
+       spool=OpenWebpageSpoolFile(1,host,path,args);
+
+       if(spool==-1)
+          mode=SpoolGet;
+       else
+         {
+          mode=SpoolPragma;
+          close(spool);
+         }
+      }
+    /* (mode==Fetch || mode==Real) are left unchanged, not modified as below. */
+   }
+ else
    {
     spool=OpenWebpageSpoolFile(1,host,path,args);
 
     if(mode==Spool && spool==-1)
        mode=SpoolGet;
-    if(mode==Fetch && spool!=-1)
-       if(RequestChanges(spool,&request))
+    else if(mode==Fetch && spool!=-1)
+      {
+       if(RequestChanges(spool,&request)==1)
           OpenWebpageSpoolFile(-2,host,path,args);
        else
           exit(0);
-    if(mode==Real && spool!=-1)
-       if(RequestChanges(spool,&request))
+      }
+    else if(mode==Real && spool!=-1)
+      {
+       if(RequestChanged && RequestChanges(spool,&request))
           OpenWebpageSpoolFile(-2,host,path,args);
        else
           mode=Spool;
+      }
+
     if(spool!=-1)
        close(spool);
    }
@@ -230,7 +286,7 @@ int wwwoffles(int online,int browser,int client)
        exit(1);
       }
    }
- else if(mode==Spool)
+ else if(mode==Spool || mode==SpoolPragma)
    {
     spool=OpenWebpageSpoolFile(1,host,path,args);
 
@@ -244,7 +300,7 @@ int wwwoffles(int online,int browser,int client)
 
  /* Set up the outgoing file. */
 
- if(mode==SpoolGet || mode==SpoolRefresh)
+ if(mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma)
    {
     outgoing=OpenOutgoingSpoolFile(0);
 
@@ -286,40 +342,81 @@ int wwwoffles(int online,int browser,int client)
 
     if(server==-1)
       {
-       if(mode==Fetch)
-          RemoteHostError(spool,url,server_host,server_port,"could not be connected.");
+       if(mode==Fetch || mode==Real)
+         {
+          lseek(spool,0,SEEK_SET);
+          ftruncate(spool,0);
+          RemoteHostError(spool,url,"the server could not be connected.");
+         }
        if(mode==Real || mode==RealNoCache)
-          RemoteHostError(client,url,server_host,server_port,"could not be connected.");
-
+          RemoteHostError(client,url,"the server could not be connected.");
        PrintMessage(Warning,"Cannot open a socket to the remote host %s port %d.",server_host,server_port);
+
        exit(1);
       }
    }
 
- /* Write request to remote server. */
+ /* Censor the header. */
 
- if(mode==Real || mode==RealNoCache)
+ if(mode==Real || mode==RealNoCache || mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma)
+   {
+    char *hostheader=(char*)malloc(strlen(host)+16);
+    char *copy=(char*)malloc(strlen(request)+strlen(host)+16);
+    char *bol,*lastbol;
+
+    sprintf(hostheader,"Host: %s\r\n",host);
+    strcpy(copy,request);
+    bol=strchr(copy,'\n')+1; *bol=0;
+    strcat(copy,hostheader);
+    bol=strchr(request,'\n')+1;
+    strcat(copy,bol);
+    free(request);
+    free(hostheader);
+    request=copy;
+
+    bol=lastbol=strchr(request,'\n')+1;
+    while((bol=strchr(bol,'\n')) && (bol-lastbol)>2)
+      {
+       bol++;
+
+       while(!strncmp("Host:",bol,5) || IsCensoredHeader(bol))
+         {
+          char *from=strchr(bol,'\n')+1,*to=bol;
+
+          *(from-2)=0;
+          if(strncmp("Host:",bol,5))
+             PrintMessage(Debug,"Censored '%s'",bol);
+
+          while(*from)
+             *to++=*from++;
+          *to=0;
+          bol-=4;
+         }
+
+       lastbol=bol;
+      }
+   }
+
+ /* Write request to remote server or outgoing file. */
+
+ if(mode==Real || mode==RealNoCache || mode==Fetch)
    {
     int err=write(server,request,strlen(request));
 
     if(err==-1)
       {
-       RemoteHostError(client,url,server_host,server_port,"could not be written to.");
+       if(mode==Real || mode==Fetch)
+         {
+          lseek(spool,0,SEEK_SET);
+          ftruncate(spool,0);
+          RemoteHostError(spool,url,"the server could not be written to.");
+         }
+       if(mode==Real || mode==RealNoCache)
+          RemoteHostError(client,url,"the server could not be written to.");
        PrintMessage(Warning,"Failed to write to remote host %s.",server_host);
        exit(1);
       }
-   }
- else if(mode==Fetch)
-   {
-    int err=write(server,request,strlen(request));
-
-    if(err==-1)
-      {
-       RemoteHostError(spool,url,server_host,server_port,"could not be written to.");
-       PrintMessage(Warning,"Failed to write to remote host %s.",server_host);
-       exit(1);
-      }
-    else
+    else if(mode==Fetch)
       {
        char *buffer=(char*)malloc(strlen(url)+16);
        sprintf(buffer,"Fetching %s\n",url);
@@ -327,7 +424,7 @@ int wwwoffles(int online,int browser,int client)
        free(buffer);
       }
    }
- else if(mode==SpoolGet || mode==SpoolRefresh)
+ else if(mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma)
    {
     int err=write(outgoing,request,strlen(request));
 
@@ -349,8 +446,14 @@ int wwwoffles(int online,int browser,int client)
 
     if(!reply)
       {
+       if(mode==Real || mode==Fetch)
+         {
+          lseek(spool,0,SEEK_SET);
+          ftruncate(spool,0);
+          RemoteHostError(spool,url,"the server timed out before sending the reply.");
+         }
        if(mode==Real || mode==RealNoCache)
-          RemoteHostError(spool,url,server_host,server_port,"timed out reading the reply.");
+          RemoteHostError(client,url,"the server timed out before sending the reply.");
        PrintMessage(Warning,"Timed out reading the reply.");
        exit(1);
       }
@@ -361,7 +464,7 @@ int wwwoffles(int online,int browser,int client)
 
        PrintMessage(Inform,"Server page has been moved.");
 
-       new_request=RequestMoved(request,reply);
+       new_request=RequestMoved(request,reply,host,path);
        if(!new_request)
           PrintMessage(Warning,"Cannot parse the reply for the new location.");
        else
@@ -407,23 +510,30 @@ int wwwoffles(int online,int browser,int client)
       }
     while(err!=-1 && (n=ReadOrTimeout(server,reply,256))>0);
 
-    if(mode==Real && (err==-1 || n<0))
-      {
-       lseek(spool,0,SEEK_SET);
-       ftruncate(spool,0);
-       if(err==-1)
+    if(mode==Real)
+       if(err==-1 || n<0)
          {
-          RemoteHostError(spool,url,server_host,server_port,"client disconnected during the transfer.");
-          PrintMessage(Warning,"Client disconnected during the transfer.");
-          exit(1);
+          lseek(spool,0,SEEK_SET);
+          ftruncate(spool,0);
+          if(err==-1)
+            {
+             RemoteHostError(spool,url,"the connection was closed by the client during the transfer.");
+             PrintMessage(Warning,"Client disconnected during the transfer.");
+             exit(1);
+            }
+          else
+            {
+             RemoteHostError(spool,url,"the server timed out during the transfer.");
+             PrintMessage(Warning,"Timed out while reading from remote host.");
+             exit(1);
+            }
          }
        else
          {
-          RemoteHostError(spool,url,server_host,server_port,"timed out during the transfer.");
-          PrintMessage(Warning,"Timed out while reading from remote host.");
-          exit(1);
+          int tell=lseek(spool,0,SEEK_CUR);
+          if(tell!=-1)
+             ftruncate(spool,tell);
          }
-      }
    }
  else if(mode==Fetch)
    {
@@ -439,10 +549,16 @@ int wwwoffles(int online,int browser,int client)
       {
        lseek(spool,0,SEEK_SET);
        ftruncate(spool,0);
-       RemoteHostError(spool,url,server_host,server_port,"timed out during the transfer.");
+       RemoteHostError(spool,url,"the server timed out during the transfer.");
        PrintMessage(Warning,"Timed out while reading from remote host.");
        exit(1);
       }
+    else
+      {
+       int tell=lseek(spool,0,SEEK_CUR);
+       if(tell!=-1)
+          ftruncate(spool,tell);
+      }
 
     if(FetchImages)
       {
@@ -474,12 +590,13 @@ int wwwoffles(int online,int browser,int client)
             }
       }
    }
- else if(mode==Spool)
+ else if(mode==Spool || mode==SpoolPragma)
    {
+    struct stat buf;
     char *head="HTTP/1.0 503 WWWOFFLE Remote Host Error\r\n"; /* This line must not be changed (see messages.c). */
     int n=read(spool,reply,64);
 
-    if(!strncmp(reply,head,strlen(head)))
+    if(!strncmp(reply,head,strlen(head)) || (!fstat(spool,&buf) && buf.st_size==0))
        OpenWebpageSpoolFile(-1,host,path,args);
 
     if(AddInfoRefresh)
@@ -497,34 +614,44 @@ int wwwoffles(int online,int browser,int client)
        if((status>=200 && status<400) && ParseHTML(spool_file,host,path) && !fstat(spool,&buf))
          {
           int position=GetHTMLEnd();
-          char *msg=(char*)malloc(strlen(url)+256);
-
-          strcpy(msg,"\n<hr>\n<p align=center>\n");
+          char *date=RFC822Date(buf.st_mtime,0),*encurl=UrlEncode(url),*localhost=GetLocalHost();
+          char *refresh=(char*)malloc(strlen(url)+2*strlen(localhost)+strlen(encurl)+128);
+          char *msg=(char*)malloc(strlen(date)+strlen(url)+2*strlen(localhost)+strlen(encurl)+256);
 
           if(!args)
-             sprintf(&msg[strlen(msg)],
-                     "WWWOFFLE - Cached %s - <a href=\"http://%s:%d/outgoing/%s/%s\">[Refresh]</a> - WWWOFFLE",
-                     RFC822Date(buf.st_mtime),GetLocalHost(),HTTP_Port,host,path[0]=='/'?"":path);
+             sprintf(refresh,
+                     "[<a href=\"http://%s:%d/refresh/%s/%s\">Refresh</a>|"
+                     "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]",
+                     localhost,HTTP_Port,host,path[0]=='/'?"":path,
+                     localhost,HTTP_Port,encurl);
           else if(*args=='!')
-             sprintf(&msg[strlen(msg)],
-                     "WWWOFFLE - Cached %s - [Posted] - WWWOFFLE",
-                     RFC822Date(buf.st_mtime));
+             sprintf(refresh,
+                     "[PostedNoRefresh]");
           else
-             sprintf(&msg[strlen(msg)],
-                     "WWWOFFLE - Cached %s - <a href=\"http://%s:%d/outgoing/%s/%s?%s\">[Refresh]</a> - WWWOFFLE",
-                     RFC822Date(buf.st_mtime),GetLocalHost(),HTTP_Port,host,path[0]=='/'?"":path,args);
+             sprintf(refresh,
+                     "[<a href=\"http://%s:%d/refresh/%s/%s?%s\">Refresh</a>|"
+                     "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]",
+                     localhost,HTTP_Port,host,path[0]=='/'?"":path,args,
+                     localhost,HTTP_Port,encurl);
 
-          strcat(msg,"\n</p>\n<hr>\n");
+          sprintf(msg,"\n"
+                      "<hr>\n"
+                      "<p align=center>\n"
+                      "WWWOFFLE - %s - %s - WWWOFFLE\n"
+                      "</p>\n"
+                      "<hr>\n",
+                  date,refresh);
 
           fseek(spool_file,0,SEEK_SET);
           while((reply=fgets_realloc(reply,spool_file)))
             {
-             if(!strncasecmp(reply,"Content-Length:",15))
+             if(!strncmp(reply,"Content-Length:",15))
                 sprintf(reply,"Content-Length: %d\r\n",atoi(&reply[16])+strlen(msg));
              write(client,reply,strlen(reply));
              if(reply[0]=='\r' || reply[0]=='\n')
                 break;
             }
+          reply=realloc(reply,257);
 
           while(position>0 && (n=fread(reply,1,position>256?256:position,spool_file))>0)
             {
@@ -539,6 +666,8 @@ int wwwoffles(int online,int browser,int client)
              write(client,reply,n);
             }
 
+          free(refresh);
+          free(encurl);
           free(msg);
          }
        else
@@ -559,26 +688,19 @@ int wwwoffles(int online,int browser,int client)
    }
  else if(mode==SpoolGet)
    {
+    int tell;
+
     WillGetURL(client,url,0);
     WillGetURL(spool,url,1);
+
+    tell=lseek(spool,0,SEEK_CUR);
+    if(tell!=-1)
+       ftruncate(spool,tell);
    }
  else if(mode==RealRefresh || mode==SpoolRefresh)
    {
     RefreshRedirect(client,url);
    }
- else /* mode==Local */
-   {
-    if(!*path && !args)
-       WelcomePage(client,PassWord?1:0);
-    else if(!strncmp("index/",path,6))
-       IndexPage(client,&path[6],args);
-    else if(!strncmp("control/",path,8))
-       ControlPage(client,&path[8],request);
-    else if(!strncmp("refresh/",path,8))
-       RefreshPage(client,&path[8],request);
-    else
-       IllegalPage(client,path,args);
-   }
 
  /* Close down and exit. */
 
